#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
from sqlalchemy import text

from .schema import create_session
from .schema.optmodel import Coauthor, Project, Projectauthor

session = create_session("OPT")


def getproject(projectid):
    project = session.query(Project).filter(text("id=:projectid")).params(projectid=projectid).first()
    session.close()
    return project


# Get the global ids for the PI and coauthors of the project
def getauthors(projectid):
    pi = (
        session.query(Projectauthor)
        .join(Project, Projectauthor.id == Project.pi)
        .filter(text("Project.id=:projectid"))
        .params(projectid=projectid)
    )
    coauthors = (
        session.query(Projectauthor)
        .join(Coauthor, Projectauthor.id == Coauthor.projectauthor_id)
        .join(Project, Coauthor.project_id == Project.id)
        .filter(text("Project.id=:projectid"))
        .params(projectid=projectid)
    )
    authors = pi.union(coauthors).all()
    session.close()
    return authors


def ispi(projectcode, globalid):
    pi = (
        session.query(Projectauthor)
        .join(Project, Projectauthor.id == Project.pi)
        .filter(text("Project.projectcode=:projectcode"))
        .filter(text("Projectauthor.globalid=:globalid"))
        .params(projectcode=projectcode)
        .params(globalid=globalid)
        .one_or_none()
    )
    if pi is None:
        return False
    else:
        return True
